#include "GLCD.h"
#include "utilities.h"

unsigned char display_buffer[Display_Width*Display_Height];

void GLCD_init(void)
{

		SET(GLCD_control_dir,GLCD_RS);//
		SET(GLCD_control_dir,GLCD_RW);
		SET(GLCD_control_dir,GLCD_EN);
		SET(GLCD_control_dir,GLCD_CS1);
		SET(GLCD_control_dir,GLCD_CS2);
		_BIT_SET(GLCD_data_dir,(0xFF<<datashift));		
		
		SET(GLCD_control_clr,GLCD_EN);
		
	// Select the Page0/Page1 and Turn on the GLCD 
    GLCD_SelectPage0();
    GLCD_send_command(0x3F);//LCD ON
		GLCD_send_command(0xC0);//START FROM LEFTMOST
		GLCD_send_command(0x8B);//LINE 0
		GLCD_send_command(0x40);//CURSOR 0
	
    GLCD_SelectPage1();
    GLCD_send_command(0x3F);//LCD ON
		GLCD_send_command(0xC0);//START FROM LEFTMOST
		GLCD_send_command(0x8B);//LINE 0
		GLCD_send_command(0x40);//CURSOR 0
		
		delay_us(1);
		GLCD_Clear_Screen();
}

void delay_us(int count)
{
  int j=0,i=0;

  for(j=0;j<count;j++)
  {
    /* At 60Mhz, the below loop introduces
    delay of APPROX 1 us */
    for(i=0;i<4;i++);
  }
}

void delay_ms(int count)
{
	int i=0,j=0;
	
	for(j=0;j<count;j++)
  {
    /* At 60Mhz, the below loop introduces
    delay of APPROX 1 ms */
    for(i=0;i<3500;i++);
  }
}

void GLCD_busy(void)
{
	CLR(GLCD_data_dir,BUSY_bit);//SET BUSY_BIT AS INPUT
	delay_us(1);
	SET(GLCD_control_set,GLCD_RW);//SET RW FOR READ MODE
	SET(GLCD_control_clr,GLCD_RS);//CLR RS FOR COMMAND MODE
	delay_us(1);
	SET(GLCD_control_set,GLCD_EN);
	delay_us(1);
	while(CHK(GLCD_data_pin,BUSY_bit));//WAIT FOR INTERNAL OPERATION
	SET(GLCD_control_clr,GLCD_EN);
	SET(GLCD_data_dir,BUSY_bit);
}

void GLCD_send_command(unsigned char command)
{		
	 GLCD_busy();//WAIT
	 SET(GLCD_control_clr,GLCD_RS);
	 SET(GLCD_control_clr,GLCD_RW);
	 _BIT_SET(GLCD_data_clr,(0xFF<<datashift));
	 _BIT_SET(GLCD_data_set,(command<<datashift));
	 delay_us(5);
	 SET(GLCD_control_set,GLCD_EN);
	 delay_us(5);
	 SET(GLCD_control_clr,GLCD_EN);
}

void GLCD_send_data(unsigned char data)
{	  
	 GLCD_busy();//WAIT
	 SET(GLCD_control_set,GLCD_RS);
	 SET(GLCD_control_clr,GLCD_RW);
	 _BIT_SET(GLCD_data_clr,(0xFF<<datashift));
	 _BIT_SET(GLCD_data_set,(data<<datashift));
	 delay_us(4);
	 SET(GLCD_control_set,GLCD_EN);
	 delay_us(4);
	 SET(GLCD_control_clr,GLCD_EN);	
}

void GLCD_8x3(const unsigned char ptr_block[],int line,int cursor)
{
	int index;
		for(index=0;index<3;index++)
		{
			display_buffer[line*128+cursor+index]|=ptr_block[index];
		}
}

void GLCD_Clear_Screen(void)
{
	int line,cursor;

	for(line=0;line<8;line++)  /* loop through all the 8lines to clear the display */
	{
		GLCD_SelectPage0();   /* Select page0 and set the cursor to first position */
		 GLCD_send_command(0x40);
		 GLCD_send_command(0xB8+line);
		for(cursor=0;cursor<128;cursor++) /* Clear all the 128 pixels of selected line */
		{
			if(cursor==64)  /* Has the cursor reached end of page0 */
			{
				GLCD_SelectPage1(); /*  then set it to beginning of page1 */
				GLCD_send_command(0x40);
				 GLCD_send_command(0xB8+line);
			}
			GLCD_send_data(0x00); /* Clear each pixel by displaying blank */
		}
	}
	GLCD_send_command(0xC0);
}




void GLCD_SetCursor(int lineNumber,int CursorPosition)
{

	int pageNumber;

	if((lineNumber <= 8) && (CursorPosition <= 128))
	{
		pageNumber = CursorPosition / 64; //Divide the cursor pos by 64 to get the page
		CursorPosition = CursorPosition % 0x3f; // Cusror pos should be within 64(0x3f),
		if(pageNumber==0x00)  /* Check for page number and set accordingly */
		{
			GLCD_SelectPage0();
		}
		else
		{
			GLCD_SelectPage1();
		}

	
		GLCD_send_command(lineNumber+0xB8); /* Command the LCD to move to specified page,line,cursor*/
		GLCD_send_command(CursorPosition+0x40);
	}
}

void GLCD_DisplayFullBitmap(const unsigned char ptr_Img[])
{
	unsigned short index;
	for(index=0;index<128*64;++index)
	{
		display_buffer[index]=ptr_Img[index];
	}
}

void GLCD_Clear_Buffer(void)
{
	unsigned short index;
	for(index=0;index<128*64;++index)
	display_buffer[index]=0x00;
}

void GLCD_DisplayBuffer()
{
	unsigned char line,cursor;
	for(line=0;line<8;++line)
	{
		GLCD_SelectPage0();
    GLCD_gotoCursor(0);
		GLCD_gotoLine(line);
		for(cursor=0;cursor<128;++cursor)
		{
			GLCD_send_data(display_buffer[128*line+cursor]);
			if(cursor==63)
			{
				GLCD_SelectPage1();
				GLCD_gotoCursor(0);
				GLCD_gotoLine(line);
			}
		}
	}
}

void GLCD_16x2(const unsigned char ptr_block[],int line,int cursor)
{
	int index,k;
	for(k=line;k<(line+2);k++)
	{
		for(index=0;index<16;index++)
		{
			display_buffer[k*128+cursor+index]=ptr_block[(k-line)*16+index];
		}
	}	
}

void GLCD_16x3(const unsigned char ptr_block[],int line,int cursor)
{
	int index,k;
	for(k=line;k<(line+3);k++)
	{
		for(index=0;index<16;index++)
		{
			display_buffer[k*128+cursor+index]=ptr_block[(k-line)*16+index];
		}
	}	
}

void GLCD_32x4(const unsigned char ptr_block[],int line,int cursor)
{
	int index,k;
	for(k=line;k<(line+4);k++)
	{
		for(index=0;index<32;index++)
		{
			display_buffer[k*128+cursor+index]=ptr_block[(k-line)*32+index];
		}
	}
}

void GLCD_32x5(const unsigned char ptr_block[],int line,int cursor)
{
	int index,k;
	for(k=line;k<(line+5);k++)
	{
		for(index=0;index<32;index++)
		{
			display_buffer[k*128+cursor+index]=ptr_block[(k-line)*32+index];
		}
	}
}

void GLCD_64x8(const unsigned char ptr_block[],char page)
{
	int index,k;
	for(k=0;k<8;k++)
	{
		for(index=0;index<64;index++)
		{
			display_buffer[k*128+page*64+index]=ptr_block[k*64+index];
		}
	}
}

void GLCD_64x8_inverted(const unsigned char ptr_block[],char page)
{
	int index,k;
	for(k=0;k<8;k++)
	{
		for(index=0;index<64;index++)
		{
			display_buffer[k*128+page*64+index]=(~ptr_block[k*64+index]);
		}
	}
}
